<!DOCTYPE html>
<html lang="ko">
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
    <meta http-equiv="X-UA-Compatible" content="IE=edge" />
    <title>dp4p bbs</title>
    <script src="https://code.jquery.com/jquery-1.12.4.min.js"></script>
    {% module Static(
        '/css/dp.css',
        '/js/dp.js',
        '/js/dp.ui.js',
        '/js/dp.helper.js', dp=True, init=True) %}
    {% module Static(
        'common.css') %}
</head>
<body>